@@ -21,27 +21,26 @@ enum class EventResult;
 
 namespace aura {
 
+class InputMethodMusDelegate;
 class InputMethodMusTestApi;
 class TextInputClientImpl;
-class Window;
 
 class AURA_EXPORT InputMethodMus : public ui::InputMethodBase {
  public:
-  using EventResultCallback = base::Callback<void(ui::mojom::EventResult)>;
+  using EventResultCallback = base::OnceCallback<void(ui::mojom::EventResult)>;
 
-  InputMethodMus(ui::internal::InputMethodDelegate* delegate, Window* window);
+  InputMethodMus(ui::internal::InputMethodDelegate* delegate,
+                 InputMethodMusDelegate* input_method_mus_delegate);
   ~InputMethodMus() override;
 
   void Init(service_manager::Connector* connector);
-  ui::EventDispatchDetails DispatchKeyEvent(
-      ui::KeyEvent* event,
-      std::unique_ptr<EventResultCallback> ack_callback) WARN_UNUSED_RESULT;
+  ui::EventDispatchDetails DispatchKeyEvent(ui::KeyEvent* event,
+                                            EventResultCallback ack_callback)
+      WARN_UNUSED_RESULT;
 
   // Overridden from ui::InputMethod:
   void OnFocus() override;
   void OnBlur() override;
-  bool OnUntranslatedIMEMessage(const base::NativeEvent& event,
-                                NativeEventResult* result) override;
   ui::EventDispatchDetails DispatchKeyEvent(ui::KeyEvent* event) override;
   void OnTextInputTypeChanged(const ui::TextInputClient* client) override;
   void OnCaretBoundsChanged(const ui::TextInputClient* client) override;
@@ -56,7 +55,7 @@ class AURA_EXPORT InputMethodMus : public ui::InputMethodBase {
   // Called from DispatchKeyEvent() to call to the InputMethod.
   ui::EventDispatchDetails SendKeyEventToInputMethod(
       const ui::KeyEvent& event,
-      std::unique_ptr<EventResultCallback> ack_callback) WARN_UNUSED_RESULT;
+      EventResultCallback ack_callback) WARN_UNUSED_RESULT;
 
   // Overridden from ui::InputMethodBase:
   void OnDidChangeFocusedClient(ui::TextInputClient* focused_before,
@@ -74,9 +73,9 @@ class AURA_EXPORT InputMethodMus : public ui::InputMethodBase {
       const ui::KeyEvent& event,
       bool handled);
 
-  // The toplevel window which is not owned by this class. This may be null
-  // for tests.
-  Window* window_;
+  // Delegate used to update window related ime state. This may be null in
+  // tests.
+  InputMethodMusDelegate* input_method_mus_delegate_;
 
   // May be null in tests.
   ui::mojom::IMEDriverPtr ime_driver_;
@@ -89,7 +88,7 @@ class AURA_EXPORT InputMethodMus : public ui::InputMethodBase {
   // Callbacks supplied to DispatchKeyEvent() are added here while awaiting
   // the response from the server. These are removed when the response is
   // received (ProcessKeyEventCallback()).
-  base::circular_deque<std::unique_ptr<EventResultCallback>> pending_callbacks_;
+  base::circular_deque<EventResultCallback> pending_callbacks_;
 
   DISALLOW_COPY_AND_ASSIGN(InputMethodMus);
 };
