@@ -4,9 +4,10 @@
 
 #include "ui/ozone/platform/drm/gpu/drm_window_proxy.h"
 
+#include "ui/gfx/gpu_fence.h"
 #include "ui/gfx/presentation_feedback.h"
+#include "ui/ozone/platform/drm/gpu/drm_overlay_plane.h"
 #include "ui/ozone/platform/drm/gpu/drm_thread.h"
-#include "ui/ozone/platform/drm/gpu/overlay_plane.h"
 #include "ui/ozone/platform/drm/gpu/proxy_helpers.h"
 #include "ui/ozone/platform/drm/gpu/scanout_buffer.h"
 
@@ -18,21 +19,25 @@ DrmWindowProxy::DrmWindowProxy(gfx::AcceleratedWidget widget,
 
 DrmWindowProxy::~DrmWindowProxy() {}
 
-void DrmWindowProxy::SchedulePageFlip(const std::vector<OverlayPlane>& planes,
-                                      SwapCompletionOnceCallback callback) {
-  auto safe_callback = CreateSafeOnceCallback(std::move(callback));
+void DrmWindowProxy::SchedulePageFlip(
+    std::vector<DrmOverlayPlane> planes,
+    SwapCompletionOnceCallback submission_callback,
+    PresentationOnceCallback presentation_callback) {
   drm_thread_->task_runner()->PostTask(
-      FROM_HERE, base::BindOnce(&DrmThread::SchedulePageFlip,
-                                base::Unretained(drm_thread_), widget_, planes,
-                                std::move(safe_callback)));
+      FROM_HERE,
+      base::BindOnce(&DrmThread::SchedulePageFlip,
+                     base::Unretained(drm_thread_), widget_,
+                     base::Passed(&planes),
+                     CreateSafeOnceCallback(std::move(submission_callback)),
+                     CreateSafeOnceCallback(std::move(presentation_callback))));
 }
 
 void DrmWindowProxy::GetVSyncParameters(
     const gfx::VSyncProvider::UpdateVSyncCallback& callback) {
   drm_thread_->task_runner()->PostTask(
-      FROM_HERE,
-      base::Bind(&DrmThread::GetVSyncParameters, base::Unretained(drm_thread_),
-                 widget_, CreateSafeCallback(callback)));
+      FROM_HERE, base::BindOnce(&DrmThread::GetVSyncParameters,
+                                base::Unretained(drm_thread_), widget_,
+                                CreateSafeCallback(callback)));
 }
 
 }  // namespace ui
