@@ -8,10 +8,13 @@
 
 #include <memory>
 
+#include "base/command_line.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_contents_observer.h"
+#include "content/public/common/content_switches.h"
+#include "content/public/test/mock_render_process_host.h"
 #include "content/public/test/test_browser_context.h"
 #include "content/public/test/test_browser_thread_bundle.h"
 #include "content/public/test/web_contents_tester.h"
@@ -30,29 +33,11 @@ namespace views {
 
 namespace {
 
-// Provides functionality to create a test WebContents.
-class WebViewTestViewsDelegate : public views::TestViewsDelegate {
- public:
-  WebViewTestViewsDelegate() {}
-  ~WebViewTestViewsDelegate() override {}
-
-  // Overriden from TestViewsDelegate.
-  content::WebContents* CreateWebContents(
-      content::BrowserContext* browser_context,
-      content::SiteInstance* site_instance) override {
-    return content::WebContentsTester::CreateTestWebContents(browser_context,
-                                                             site_instance);
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(WebViewTestViewsDelegate);
-};
-
-// Provides functionality to observe events on a WebContents like WasShown/
-// WasHidden/WebContentsDestroyed.
+// Provides functionality to observe events on a WebContents like
+// OnVisibilityChanged/WebContentsDestroyed.
 class WebViewTestWebContentsObserver : public content::WebContentsObserver {
  public:
-  WebViewTestWebContentsObserver(content::WebContents* web_contents)
+  explicit WebViewTestWebContentsObserver(content::WebContents* web_contents)
       : web_contents_(web_contents),
         was_shown_(false),
         shown_count_(0),
@@ -72,18 +57,27 @@ class WebViewTestWebContentsObserver : public content::WebContentsObserver {
     web_contents_ = NULL;
   }
 
-  void WasShown() override {
+  void OnVisibilityChanged(content::Visibility visibility) override {
+    switch (visibility) {
+      case content::Visibility::VISIBLE: {
 #if defined(USE_AURA)
-    valid_root_while_shown_ =
-        web_contents()->GetNativeView()->GetRootWindow() != NULL;
+        valid_root_while_shown_ =
+            web_contents()->GetNativeView()->GetRootWindow() != NULL;
 #endif
-    was_shown_ = true;
-    ++shown_count_;
-  }
-
-  void WasHidden() override {
-    was_shown_ = false;
-    ++hidden_count_;
+        was_shown_ = true;
+        ++shown_count_;
+        break;
+      }
+      case content::Visibility::HIDDEN: {
+        was_shown_ = false;
+        ++hidden_count_;
+        break;
+      }
+      default: {
+        ADD_FAILURE() << "Unexpected call to OnVisibilityChanged.";
+        break;
+      }
+    }
   }
 
   bool was_shown() const { return was_shown_; }
@@ -134,14 +128,28 @@ class WebViewUnitTest : public views::test::WidgetTest {
 
   ~WebViewUnitTest() override {}
 
+  std::unique_ptr<content::WebContents> CreateWebContentsForWebView(
+      content::BrowserContext* browser_context) {
+    return content::WebContentsTester::CreateTestWebContents(browser_context,
+                                                             nullptr);
+  }
+
   void SetUp() override {
-    set_views_delegate(base::WrapUnique(new WebViewTestViewsDelegate));
+    views::WebView::WebContentsCreator creator = base::BindRepeating(
+        &WebViewUnitTest::CreateWebContentsForWebView, base::Unretained(this));
+    scoped_web_contents_creator_ =
+        std::make_unique<views::WebView::ScopedWebContentsCreatorForTesting>(
+            creator);
+    set_views_delegate(base::WrapUnique(new views::TestViewsDelegate));
     browser_context_.reset(new content::TestBrowserContext);
     WidgetTest::SetUp();
     // Set the test content browser client to avoid pulling in needless
     // dependencies from content.
     SetBrowserClientForTesting(&test_browser_client_);
 
+    base::CommandLine::ForCurrentProcess()->AppendSwitch(
+        switches::kDisableBackgroundingOccludedWindowsForTesting);
+
     // Create a top level widget and add a child, and give it a WebView as a
     // child.
     top_level_widget_ = CreateTopLevelFramelessPlatformWidget();
@@ -156,6 +164,7 @@ class WebViewUnitTest : public views::test::WidgetTest {
   }
 
   void TearDown() override {
+    scoped_web_contents_creator_.reset();
     top_level_widget_->Close();  // Deletes all children and itself.
     RunPendingMessages();
 
@@ -172,14 +181,16 @@ class WebViewUnitTest : public views::test::WidgetTest {
   NativeViewHost* holder() const { return web_view_->holder_; }
 
   std::unique_ptr<content::WebContents> CreateWebContents() const {
-    return base::WrapUnique(content::WebContents::Create(
-        content::WebContents::CreateParams(browser_context_.get())));
+    return content::WebContents::Create(
+        content::WebContents::CreateParams(browser_context_.get()));
   }
 
  private:
   content::TestBrowserThreadBundle test_browser_thread_bundle_;
   std::unique_ptr<content::TestBrowserContext> browser_context_;
   content::TestContentBrowserClient test_browser_client_;
+  std::unique_ptr<views::WebView::ScopedWebContentsCreatorForTesting>
+      scoped_web_contents_creator_;
 
   Widget* top_level_widget_ = nullptr;
   WebView* web_view_ = nullptr;
@@ -245,9 +256,7 @@ TEST_F(WebViewUnitTest, TestWebViewAttachDetachWebContents) {
   // parent does not make the web contents visible.
   top_level_widget()->Hide();
   web_view()->SetVisible(true);
-  // TODO(tapted): The following line is wrong, the shown_count() should still
-  // be 1, until the parent window is made visible on the line after.
-  EXPECT_EQ(2, observer1.shown_count());
+  EXPECT_EQ(1, observer1.shown_count());
   top_level_widget()->Show();
   EXPECT_EQ(2, observer1.shown_count());
   top_level_widget()->Hide();
@@ -315,19 +324,31 @@ TEST_F(WebViewUnitTest, EmbeddedFullscreenDuringScreenCapture_Layout) {
   delegate.set_is_fullscreened(true);
   static_cast<content::WebContentsObserver*>(web_view())->
       DidToggleFullscreenModeForTab(true, false);
-  EXPECT_EQ(gfx::Rect(18, 26, 64, 48), holder()->bounds());
+
+  // The expected size should be scaled to whichever dimension matches the
+  // holder first, with the other scaled from the capture size to match the
+  // holder.  So 100, 100 holder size and 64, 48 capture size gives:
+  // 100 / 64 * 48 = 75
+  // The positioning centers the unmatched holder/capture dimension, giving:
+  // (100 - 75 = 25) / 2 = 12
+  EXPECT_EQ(gfx::Rect(0, 12, 100, 75), holder()->bounds());
 
   // Resize the WebView so that its width is smaller than the capture width.
   // Expect the holder to be scaled-down, letterboxed style.
-  web_view()->SetBoundsRect(gfx::Rect(0, 0, 32, 32));
-  EXPECT_EQ(gfx::Rect(0, 4, 32, 24), holder()->bounds());
+  web_view()->SetBoundsRect(gfx::Rect(0, 0, 32, 100));
+  EXPECT_EQ(gfx::Rect(0, 38, 32, 24), holder()->bounds());
+
+  // Resize the WebView so that its height is smaller than the capture height.
+  // Expect the holder to be scaled-down, pillarboxed style.
+  web_view()->SetBoundsRect(gfx::Rect(0, 0, 100, 24));
+  EXPECT_EQ(gfx::Rect(34, 0, 32, 24), holder()->bounds());
 
   // Transition back out of fullscreen mode a final time and confirm the bounds
   // of the holder fill the entire WebView once again.
   delegate.set_is_fullscreened(false);
   static_cast<content::WebContentsObserver*>(web_view())->
       DidToggleFullscreenModeForTab(false, false);
-  EXPECT_EQ(gfx::Rect(0, 0, 32, 32), holder()->bounds());
+  EXPECT_EQ(gfx::Rect(0, 0, 100, 24), holder()->bounds());
 }
 
 // Tests that a WebView correctly switches between WebContentses when one of
@@ -361,7 +382,7 @@ TEST_F(WebViewUnitTest, EmbeddedFullscreenDuringScreenCapture_Switching) {
   static_cast<content::WebContentsObserver*>(web_view())->
       DidToggleFullscreenModeForTab(true, false);
   EXPECT_EQ(web_contents1->GetNativeView(), holder()->native_view());
-  EXPECT_EQ(gfx::Rect(18, 26, 64, 48), holder()->bounds());
+  EXPECT_EQ(gfx::Rect(0, 12, 100, 75), holder()->bounds());
 
   // When setting the WebContents to nullptr, the native view should become
   // unset.
@@ -372,7 +393,7 @@ TEST_F(WebViewUnitTest, EmbeddedFullscreenDuringScreenCapture_Switching) {
   // instance, expect the native view and layout to reflect that.
   web_view()->SetWebContents(web_contents1.get());
   EXPECT_EQ(web_contents1->GetNativeView(), holder()->native_view());
-  EXPECT_EQ(gfx::Rect(18, 26, 64, 48), holder()->bounds());
+  EXPECT_EQ(gfx::Rect(0, 12, 100, 75), holder()->bounds());
 
   // Now, switch to a different, non-null WebContents instance and check that
   // the native view has changed and the holder is filling WebView again.
@@ -383,7 +404,7 @@ TEST_F(WebViewUnitTest, EmbeddedFullscreenDuringScreenCapture_Switching) {
   // Finally, switch back to the first WebContents (still fullscreened).
   web_view()->SetWebContents(web_contents1.get());
   EXPECT_EQ(web_contents1->GetNativeView(), holder()->native_view());
-  EXPECT_EQ(gfx::Rect(18, 26, 64, 48), holder()->bounds());
+  EXPECT_EQ(gfx::Rect(0, 12, 100, 75), holder()->bounds());
 }
 
 // Tests that clicking anywhere within the bounds of WebView, and either outside
@@ -413,7 +434,7 @@ TEST_F(WebViewUnitTest, EmbeddedFullscreenDuringScreenCapture_ClickToFocus) {
   delegate.set_is_fullscreened(true);
   static_cast<content::WebContentsObserver*>(web_view())->
       DidToggleFullscreenModeForTab(true, false);
-  EXPECT_EQ(gfx::Rect(18, 21, 64, 48), holder()->bounds());
+  EXPECT_EQ(gfx::Rect(0, 7, 100, 75), holder()->bounds());
 
   // Focus the other widget.
   something_to_focus->RequestFocus();
@@ -475,4 +496,58 @@ TEST_F(WebViewUnitTest, DetachedWebViewDestructor) {
   webview.reset();
 }
 
+// Test that the specified crashed overlay view is shown when a WebContents
+// is in a crashed state.
+TEST_F(WebViewUnitTest, CrashedOverlayView) {
+  const std::unique_ptr<content::WebContents> web_contents(CreateWebContents());
+  std::unique_ptr<WebView> web_view(
+      new WebView(web_contents->GetBrowserContext()));
+  View* contents_view = top_level_widget()->GetContentsView();
+  contents_view->AddChildView(web_view.get());
+  web_view->SetWebContents(web_contents.get());
+
+  View* crashed_overlay_view = new View();
+  web_view->SetCrashedOverlayView(crashed_overlay_view);
+  EXPECT_FALSE(crashed_overlay_view->IsDrawn());
+
+  // Normally when a renderer crashes, the WebView will learn about it
+  // automatically via WebContentsObserver. Since this is a test
+  // WebContents, simulate that by calling SetIsCrashed and then
+  // explicitly calling RenderViewDeleted on the WebView to trigger it
+  // to swap in the crashed overlay view.
+  web_contents->SetIsCrashed(base::TERMINATION_STATUS_PROCESS_CRASHED, -1);
+  EXPECT_TRUE(web_contents->IsCrashed());
+  static_cast<content::WebContentsObserver*>(web_view.get())
+      ->RenderViewDeleted(nullptr);
+  EXPECT_TRUE(crashed_overlay_view->IsDrawn());
+}
+
+// Test that a crashed overlay view isn't deleted if it's owned by client.
+TEST_F(WebViewUnitTest, CrashedOverlayViewOwnedbyClient) {
+  const std::unique_ptr<content::WebContents> web_contents(CreateWebContents());
+  std::unique_ptr<WebView> web_view(
+      new WebView(web_contents->GetBrowserContext()));
+  View* contents_view = top_level_widget()->GetContentsView();
+  contents_view->AddChildView(web_view.get());
+  web_view->SetWebContents(web_contents.get());
+
+  View* crashed_overlay_view = new View();
+  crashed_overlay_view->set_owned_by_client();
+  web_view->SetCrashedOverlayView(crashed_overlay_view);
+  EXPECT_FALSE(crashed_overlay_view->IsDrawn());
+
+  // Simulate a renderer crash (see above).
+  web_contents->SetIsCrashed(base::TERMINATION_STATUS_PROCESS_CRASHED, -1);
+  EXPECT_TRUE(web_contents->IsCrashed());
+  static_cast<content::WebContentsObserver*>(web_view.get())
+      ->RenderViewDeleted(nullptr);
+  EXPECT_TRUE(crashed_overlay_view->IsDrawn());
+
+  web_view->SetCrashedOverlayView(nullptr);
+  web_view.reset();
+
+  // This shouldn't crash, we still own this.
+  delete crashed_overlay_view;
+}
+
 }  // namespace views
