@@ -9,7 +9,7 @@
  */
 
 /**
- * Combinaiton of CrOnc.VPNType + AuthenticationType for IPsec.
+ * Combination of CrOnc.VPNType + AuthenticationType for IPsec.
  * Note: closure does not always recognize this if inside function() {}.
  * @enum {string}
  */
@@ -24,6 +24,8 @@ var VPNConfigType = {
 
 /** @const */ var DEFAULT_HASH = 'default';
 /** @const */ var DO_NOT_CHECK_HASH = 'do-not-check';
+/** @const */ var NO_CERTS_HASH = 'no-certs';
+/** @const */ var NO_USER_CERT_HASH = 'no-user-cert';
 
 Polymer({
   is: 'network-config',
@@ -53,9 +55,6 @@ Polymer({
      */
     type: String,
 
-    /** Set by embedder if saveOrConnect should always connect. */
-    connectOnSave: Boolean,
-
     /** True if the user configuring the network can toggle the shared state. */
     shareAllowEnable: Boolean,
 
@@ -129,7 +128,7 @@ Polymer({
       },
     },
 
-    /** @private */
+    /** @private {string|undefined} */
     selectedServerCaHash_: String,
 
     /**
@@ -143,7 +142,7 @@ Polymer({
       },
     },
 
-    /** @private */
+    /** @private {string|undefined} */
     selectedUserCertHash_: String,
 
     /**
@@ -280,7 +279,7 @@ Polymer({
 
     /**
      * Array of values for the VPN Type dropdown. For L2TP-IPSec, the
-     * IPsec AuthenticationType ('PSK' or 'Cert') is incuded in the type.
+     * IPsec AuthenticationType ('PSK' or 'Cert') is included in the type.
      * Note: closure does not recognize Array<VPNConfigType> here.
      * @private {!Array<string>}
      * @const
@@ -313,6 +312,7 @@ Polymer({
     'updateIsConfigured_(configProperties_, eapProperties_.*)',
     'updateIsConfigured_(configProperties_.WiFi.*)',
     'updateIsConfigured_(configProperties_.VPN.*, vpnType_)',
+    'updateIsConfigured_(selectedUserCertHash_)',
   ],
 
   /** @const */
@@ -343,6 +343,8 @@ Polymer({
 
   init: function() {
     this.propertiesSent_ = false;
+    this.selectedServerCaHash_ = undefined;
+    this.selectedUserCertHash_ = undefined;
     this.guid = this.networkProperties.GUID;
     this.type = this.networkProperties.Type;
     if (this.guid) {
@@ -360,7 +362,19 @@ Polymer({
     this.setShareNetwork_();
   },
 
-  saveOrConnect: function() {
+  save: function() {
+    this.saveAndConnect_(false /* connect */);
+  },
+
+  connect: function() {
+    this.saveAndConnect_(true /* connect */);
+  },
+
+  /**
+   * @param {boolean} connect If true, connect after save.
+   * @private
+   */
+  saveAndConnect_: function(connect) {
     if (this.propertiesSent_)
       return;
     this.propertiesSent_ = true;
@@ -376,14 +390,14 @@ Polymer({
            this.globalPolicy.AllowOnlyPolicyNetworksToConnect)) {
         CrOnc.setTypeProperty(propertiesToSet, 'AutoConnect', false);
       }
-      // Create the configuration, then connect to it in the callback.
       this.networkingPrivate.createNetwork(
-          this.shareNetwork_, propertiesToSet,
-          this.createNetworkCallback_.bind(this));
+          this.shareNetwork_, propertiesToSet, (guid) => {
+            this.createNetworkCallback_(connect, guid);
+          });
     } else {
-      propertiesToSet.GUID = this.guid;
-      this.networkingPrivate.setProperties(
-          this.guid, propertiesToSet, this.setPropertiesCallback_.bind(this));
+      this.networkingPrivate.setProperties(this.guid, propertiesToSet, () => {
+        this.setPropertiesCallback_(connect);
+      });
     }
   },
 
@@ -391,7 +405,8 @@ Polymer({
   focusFirstInput_: function() {
     Polymer.dom.flush();
     var e = this.$$(
-        'network-config-input:not([disabled]),' +
+        'cr-input:not([readonly]),' +
+        'network-password-input:not([disabled]),' +
         'network-config-select:not([disabled])');
     if (e)
       e.focus();
@@ -401,7 +416,7 @@ Polymer({
   connectIfConfigured_: function() {
     if (!this.isConfigured_)
       return;
-    this.saveOrConnect();
+    this.connect();
   },
 
   /** @private */
@@ -432,19 +447,39 @@ Polymer({
   /** @private */
   onCertificateListsChanged_: function() {
     this.networkingPrivate.getCertificateLists(function(certificateLists) {
-      var caCerts = [this.getDefaultCert_(
-          this.i18n('networkCAUseDefault'), DEFAULT_HASH)];
-      caCerts = caCerts.concat(certificateLists.serverCaCertificates);
+      var isOpenVpn = this.type == CrOnc.Type.VPN &&
+          this.get('VPN.Type', this.configProperties_) ==
+              CrOnc.VPNType.OPEN_VPN;
+
+      var caCerts = certificateLists.serverCaCertificates.slice();
+      if (!isOpenVpn) {
+        // 'Default' is the same as 'Do not check' except it sets
+        // eap.UseSystemCAs (which does not apply to OpenVPN).
+        caCerts.unshift(this.getDefaultCert_(
+            this.i18n('networkCAUseDefault'), DEFAULT_HASH));
+      }
       caCerts.push(this.getDefaultCert_(
           this.i18n('networkCADoNotCheck'), DO_NOT_CHECK_HASH));
       this.set('serverCaCerts_', caCerts);
 
       var userCerts = certificateLists.userCertificates.slice();
+      // Only hardware backed user certs are supported.
+      userCerts.forEach(function(cert) {
+        if (!cert.hardwareBacked)
+          cert.hash = '';  // Clear the hash to invalidate the certificate.
+      });
+      if (isOpenVpn) {
+        // OpenVPN allows but does not require a user certificate.
+        userCerts.unshift(this.getDefaultCert_(
+            this.i18n('networkNoUserCert'), NO_USER_CERT_HASH));
+      }
       if (!userCerts.length) {
         userCerts = [this.getDefaultCert_(
-            this.i18n('networkCertificateNoneInstalled'), '')];
+            this.i18n('networkCertificateNoneInstalled'), NO_CERTS_HASH)];
       }
       this.set('userCerts_', userCerts);
+
+      this.updateSelectedCerts_();
       this.updateCertError_();
     }.bind(this));
   },
@@ -501,8 +536,9 @@ Polymer({
     this.propertiesReceived_ = true;
     this.networkProperties = properties;
     this.setError_(properties.ErrorState);
+    this.updateCertError_();
 
-    // Set the current shareNetwork_ value when porperties are received.
+    // Set the current shareNetwork_ value when properties are received.
     this.setShareNetwork_();
   },
 
@@ -727,7 +763,7 @@ Polymer({
     } else {
       this.set('eapProperties_.Inner', undefined);
     }
-    // Set the share vaule to its default when the EAP.Outer value changes.
+    // Set the share value to its default when the EAP.Outer value changes.
     this.setShareNetwork_();
   },
 
@@ -869,6 +905,7 @@ Polymer({
         break;
     }
     this.updateCertError_();
+    this.onCertificateListsChanged_();
   },
 
   /** @private */
@@ -904,16 +941,39 @@ Polymer({
 
   /** @private */
   updateCertError_: function() {
-    /** @const */ var certError = 'networkErrorNoUserCertificate';
-    if (this.error && this.error != certError)
+    // If |this.error| was set to something other than a cert error, do not
+    // change it.
+    /** @const */ var noCertsError = 'networkErrorNoUserCertificate';
+    /** @const */ var noValidCertsError = 'networkErrorNotHardwareBacked';
+    if (this.error && this.error != noCertsError &&
+        this.error != noValidCertsError) {
       return;
+    }
 
     var requireCerts = (this.showEap_ && this.showEap_.UserCert) ||
         (this.showVpn_ && this.showVpn_.UserCert);
-    this.setError_(requireCerts && !this.userCerts_.length ? certError : '');
+    if (!requireCerts) {
+      this.setError_('');
+      return;
+    }
+    if (!this.userCerts_.length || this.userCerts_[0].hash == NO_CERTS_HASH) {
+      this.setError_(noCertsError);
+      return;
+    }
+    var validUserCert = this.userCerts_.find(function(cert) {
+      return !!cert.hash;
+    });
+    if (!validUserCert) {
+      this.setError_(noValidCertsError);
+      return;
+    }
+    this.setError_('');
+    return;
   },
 
   /**
+   * Sets the selected cert if |pem| (serverCa) or |certId| (user) is specified.
+   * Otherwise sets a default value if no certificate is selected.
    * @param {string|undefined} pem
    * @param {string|undefined} certId
    * @private
@@ -926,8 +986,6 @@ Polymer({
       if (serverCa)
         this.selectedServerCaHash_ = serverCa.hash;
     }
-    if (!this.selectedServerCaHash_ && this.serverCaCerts_[0])
-      this.selectedServerCaHash_ = this.serverCaCerts_[0].hash;
 
     if (certId) {
       var userCert = this.userCerts_.find(function(cert) {
@@ -936,6 +994,38 @@ Polymer({
       if (userCert)
         this.selectedUserCertHash_ = userCert.hash;
     }
+    this.updateSelectedCerts_();
+    this.updateIsConfigured_();
+  },
+
+  /**
+   * @param {!Array<!chrome.networkingPrivate.Certificate>} certs
+   * @param {string|undefined} hash
+   * @private
+   * @return {!chrome.networkingPrivate.Certificate|undefined}
+   */
+  findCert_: function(certs, hash) {
+    if (!hash)
+      return undefined;
+    return certs.find((cert) => {
+      return cert.hash == hash;
+    });
+  },
+
+  /**
+   * Called when the certificate list or a selected certificate changes.
+   * Ensures that each selected certificate exists in its list, or selects the
+   * correct default value.
+   * @private
+   */
+  updateSelectedCerts_: function() {
+    if (!this.findCert_(this.serverCaCerts_, this.selectedServerCaHash_))
+      this.selectedServerCaHash_ = undefined;
+    if (!this.selectedServerCaHash_ && this.serverCaCerts_[0])
+      this.selectedServerCaHash_ = this.serverCaCerts_[0].hash;
+
+    if (!this.findCert_(this.userCerts_, this.selectedUserCertHash_))
+      this.selectedUserCertHash_ = undefined;
     if (!this.selectedUserCertHash_ && this.userCerts_[0])
       this.selectedUserCertHash_ = this.userCerts_[0].hash;
   },
@@ -945,6 +1035,9 @@ Polymer({
    * @private
    */
   getIsConfigured_: function() {
+    if (!this.configProperties_)
+      return false;
+
     if (this.configProperties_.Type == CrOnc.Type.VPN)
       return this.vpnIsConfigured_();
 
@@ -1059,6 +1152,15 @@ Polymer({
     return true;
   },
 
+  /**
+   * @return {boolean}
+   * @private
+   */
+  selectedUserCertHashIsValid_: function() {
+    return !!this.selectedUserCertHash_ &&
+        this.selectedUserCertHash_ != NO_CERTS_HASH;
+  },
+
   /**
    * @return {boolean}
    * @private
@@ -1069,7 +1171,7 @@ Polymer({
       return false;
     if (eap.Outer != CrOnc.EAPType.EAP_TLS)
       return true;
-    return !!this.selectedUserCertHash_;
+    return this.selectedUserCertHashIsValid_();
   },
 
   /**
@@ -1085,10 +1187,13 @@ Polymer({
       case VPNConfigType.L2TP_IPSEC_PSK:
         return !!this.get('L2TP.Username', vpn) && !!this.get('IPsec.PSK', vpn);
       case VPNConfigType.L2TP_IPSEC_CERT:
-        return !!this.get('L2TP.Username', vpn) && !!this.selectedUserCertHash_;
+        return !!this.get('L2TP.Username', vpn) &&
+            this.selectedUserCertHashIsValid_();
       case VPNConfigType.OPEN_VPN:
-        return !!this.get('OpenVPN.Username', vpn) &&
-            !!this.selectedUserCertHash_;
+        // OpenVPN should require username + password OR a user cert. However,
+        // there may be servers with different requirements so err on the side
+        // of permissiveness.
+        return true;
     }
     return false;
   },
@@ -1096,6 +1201,11 @@ Polymer({
   /** @private */
   getPropertiesToSet_: function() {
     var propertiesToSet = Object.assign({}, this.configProperties_);
+    // Do not set AutoConnect by default, the connection manager will set
+    // it to true on a successful connection.
+    CrOnc.setTypeProperty(propertiesToSet, 'AutoConnect', undefined);
+    if (this.guid)
+      propertiesToSet.GUID = this.guid;
     var eap = this.getEap_(propertiesToSet);
     if (eap)
       this.setEapProperties_(eap);
@@ -1116,9 +1226,7 @@ Polymer({
     var caHash = this.selectedServerCaHash_ || '';
     if (!caHash || caHash == DO_NOT_CHECK_HASH || caHash == DEFAULT_HASH)
       return [];
-    var serverCa = this.serverCaCerts_.find(function(cert) {
-      return cert.hash == caHash;
-    });
+    var serverCa = this.findCert_(this.serverCaCerts_, caHash);
     return serverCa && serverCa.pem ? [serverCa.pem] : [];
   },
 
@@ -1127,12 +1235,12 @@ Polymer({
    * @private
    */
   getUserCertPkcs11Id_: function() {
-    var userHash = this.selectedUserCertHash_;
-    if (!userHash)
+    var userCertHash = this.selectedUserCertHash_ || '';
+    if (!this.selectedUserCertHashIsValid_() ||
+        userCertHash == NO_USER_CERT_HASH) {
       return '';
-    var userCert = this.userCerts_.find(function(cert) {
-      return cert.hash == userHash;
-    });
+    }
+    var userCert = this.findCert_(this.userCerts_, userCertHash);
     return (userCert && userCert.PKCS11Id) || '';
   },
 
@@ -1203,8 +1311,11 @@ Polymer({
     return (chrome.runtime.lastError && chrome.runtime.lastError.message) || '';
   },
 
-  /** @private */
-  setPropertiesCallback_: function() {
+  /**
+   * @param {boolean} connect If true, connect after save.
+   * @private
+   */
+  setPropertiesCallback_: function(connect) {
     this.setError_(this.getRuntimeError_());
     if (this.error) {
       console.error('setProperties error: ' + this.guid + ': ' + this.error);
@@ -1212,7 +1323,7 @@ Polymer({
       return;
     }
     var connectState = this.networkProperties.ConnectionState;
-    if (this.connectOnSave &&
+    if (connect &&
         (!connectState ||
          connectState == CrOnc.ConnectionState.NOT_CONNECTED)) {
       this.startConnect_(this.guid);
@@ -1222,10 +1333,11 @@ Polymer({
   },
 
   /**
+   * @param {boolean} connect If true, connect after save.
    * @param {string} guid
    * @private
    */
-  createNetworkCallback_: function(guid) {
+  createNetworkCallback_: function(connect, guid) {
     this.setError_(this.getRuntimeError_());
     if (this.error) {
       console.error(
@@ -1234,7 +1346,8 @@ Polymer({
       this.propertiesSent_ = false;
       return;
     }
-    this.startConnect_(guid);
+    if (connect)
+      this.startConnect_(guid);
   },
 
   /**
