@@ -39,7 +39,9 @@
 
 #include "qwebengineurlscheme.h"
 
-#include <url/url_util_qt.h>
+#include "url/url_util_qt.h"
+
+#include <QtDebug>
 
 QT_BEGIN_NAMESPACE
 
@@ -59,6 +61,8 @@ ASSERT_ENUMS_MATCH(QWebEngineUrlScheme::ServiceWorkersAllowed, url::CustomScheme
 ASSERT_ENUMS_MATCH(QWebEngineUrlScheme::ViewSourceAllowed, url::CustomScheme::ViewSourceAllowed)
 ASSERT_ENUMS_MATCH(QWebEngineUrlScheme::ContentSecurityPolicyIgnored, url::CustomScheme::ContentSecurityPolicyIgnored)
 
+static bool g_schemesLocked = false;
+
 class QWebEngineUrlSchemePrivate : public QSharedData
                                  , public url::CustomScheme
 {
@@ -84,7 +88,7 @@ public:
   URLs.
 
   Custom URL schemes must be configured early at application startup, before
-  creating any Qt WebEngine classes. In general this means the schemes need to be configured before
+  creating any \QWE classes. In general this means the schemes need to be configured before
   a QGuiApplication or QApplication instance is created.
 
   Every registered scheme configuration applies globally to all profiles.
@@ -357,6 +361,33 @@ void QWebEngineUrlScheme::setFlags(Flags newValue)
 */
 void QWebEngineUrlScheme::registerScheme(const QWebEngineUrlScheme &scheme)
 {
+    if (scheme.d->name.empty()) {
+        qWarning() << "QWebEngineUrlScheme::registerScheme: Scheme name cannot be empty";
+        return;
+    }
+
+    bool needsPort = scheme.d->has_port_component();
+    bool hasPort = scheme.d->default_port != url::PORT_UNSPECIFIED;
+    if (needsPort && !hasPort) {
+        qWarning() << "QWebEngineUrlScheme::registerScheme: Scheme" << scheme.name() << "needs a default port";
+        return;
+    }
+
+    if (url::CustomScheme::FindScheme(scheme.d->name)) {
+        qWarning() << "QWebEngineUrlScheme::registerScheme: Scheme" << scheme.name() << "already registered";
+        return;
+    }
+
+    if (url::IsStandard(scheme.d->name.data(), url::Component(0, scheme.d->name.size()))) {
+        qWarning() << "QWebEngineUrlScheme::registerScheme: Scheme" << scheme.name() << "is a standard scheme";
+        return;
+    }
+
+    if (g_schemesLocked) {
+        qWarning() << "QWebEngineUrlScheme::registerScheme: Too late to register scheme" << scheme.name();
+        return;
+    }
+
     url::CustomScheme::AddScheme(*scheme.d);
 }
 
@@ -374,4 +405,9 @@ QWebEngineUrlScheme QWebEngineUrlScheme::schemeByName(const QByteArray &name)
     return QWebEngineUrlScheme();
 }
 
+void QWebEngineUrlScheme::lockSchemes()
+{
+    g_schemesLocked = true;
+}
+
 QT_END_NAMESPACE
