@@ -43,6 +43,8 @@
 #include "profile_adapter.h"
 #include "qquickwebengineprofile_p.h"
 
+#include "QFileInfo"
+
 using QtWebEngineCore::ProfileAdapterClient;
 
 QT_BEGIN_NAMESPACE
@@ -135,7 +137,7 @@ QQuickWebEngineDownloadItemPrivate::~QQuickWebEngineDownloadItemPrivate()
     \l{WebEngineProfile::downloadRequested}{downloadRequested} signal together
     with a newly created WebEngineDownloadItem. The application can then examine
     this item and decide whether to accept it or not. A signal handler must
-    explicitly call accept() on the item for Qt WebEngine to actually start
+    explicitly call accept() on the item for \QWE to actually start
     downloading and writing data to disk. If no signal handler calls accept(),
     then the download request will be automatically rejected and nothing will be
     written to disk.
@@ -154,7 +156,7 @@ QQuickWebEngineDownloadItemPrivate::~QQuickWebEngineDownloadItemPrivate()
     \section2 Web Page Downloads
 
     In addition to normal file downloads, which consist simply of retrieving
-    some raw bytes from the network and writing them to disk, Qt WebEngine also
+    some raw bytes from the network and writing them to disk, \QWE also
     supports saving complete web pages, which involves parsing the page's HTML,
     downloading any dependent resources, and potentially packaging everything
     into a special file format (\l savePageFormat). To check if a download is
@@ -427,6 +429,16 @@ void QQuickWebEngineDownloadItem::setPath(QString path)
         return;
     }
     if (d->downloadPath != path) {
+        if (QFileInfo(path).fileName().isEmpty()) {
+            qWarning("The download path does not include file name.");
+            return;
+        }
+
+        if (QFileInfo(path).isDir()) {
+            qWarning("The download path matches with an already existing directory path.");
+            return;
+        }
+
         d->downloadPath = path;
         Q_EMIT pathChanged();
     }
