@@ -189,7 +189,7 @@ bool QQuickWebEngineViewPrivate::profileInitialized() const
     return m_profileInitialized;
 }
 
-void QQuickWebEngineViewPrivate::destroy()
+void QQuickWebEngineViewPrivate::releaseProfile()
 {
     // The profile for this web contents is about to be
     // garbage collected, delete WebContents first and
@@ -333,12 +333,16 @@ void QQuickWebEngineViewPrivate::runFileChooser(QSharedPointer<FilePickerControl
         ui()->showFilePicker(controller);
 }
 
-void QQuickWebEngineViewPrivate::passOnFocus(bool reverse)
+bool QQuickWebEngineViewPrivate::passOnFocus(bool reverse)
 {
     Q_Q(QQuickWebEngineView);
     // The child delegate currently has focus, find the next one from there and give it focus.
     QQuickItem *next = q->scopedFocusItem()->nextItemInFocusChain(!reverse);
-    next->forceActiveFocus(reverse ? Qt::BacktabFocusReason : Qt::TabFocusReason);
+    if (next) {
+        next->forceActiveFocus(reverse ? Qt::BacktabFocusReason : Qt::TabFocusReason);
+        return true;
+    }
+    return false;
 }
 
 void QQuickWebEngineViewPrivate::titleChanged(const QString &title)
@@ -927,6 +931,16 @@ void QQuickWebEngineViewPrivate::updateAction(QQuickWebEngineView::WebAction act
     case QQuickWebEngineView::ViewSource:
         enabled = adapter->canViewSource();
         break;
+    case QQuickWebEngineView::Cut:
+    case QQuickWebEngineView::Copy:
+    case QQuickWebEngineView::Paste:
+    case QQuickWebEngineView::Undo:
+    case QQuickWebEngineView::Redo:
+    case QQuickWebEngineView::SelectAll:
+    case QQuickWebEngineView::PasteAndMatchStyle:
+    case QQuickWebEngineView::Unselect:
+        enabled = adapter->hasFocusedFrame();
+        break;
     default:
         break;
     }
@@ -944,6 +958,18 @@ void QQuickWebEngineViewPrivate::updateNavigationActions()
     updateAction(QQuickWebEngineView::ViewSource);
 }
 
+void QQuickWebEngineViewPrivate::updateEditActions()
+{
+    updateAction(QQuickWebEngineView::Cut);
+    updateAction(QQuickWebEngineView::Copy);
+    updateAction(QQuickWebEngineView::Paste);
+    updateAction(QQuickWebEngineView::Undo);
+    updateAction(QQuickWebEngineView::Redo);
+    updateAction(QQuickWebEngineView::SelectAll);
+    updateAction(QQuickWebEngineView::PasteAndMatchStyle);
+    updateAction(QQuickWebEngineView::Unselect);
+}
+
 QUrl QQuickWebEngineView::url() const
 {
     Q_D(const QQuickWebEngineView);
@@ -1277,7 +1303,7 @@ void QQuickWebEngineView::setBackgroundColor(const QColor &color)
 
 /*!
     \property QQuickWebEngineView::audioMuted
-    \brief the state of whether the current page audio is muted.
+    \brief The state of whether the current page audio is muted.
     \since 5.7
 
     The default value is false.
@@ -1844,6 +1870,7 @@ QQuickWebEngineAction *QQuickWebEngineView::action(WebAction action)
         break;
     case Stop:
         text = tr("Stop");
+        iconName = QStringLiteral("process-stop");
         break;
     case Reload:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Reload);
@@ -1851,28 +1878,35 @@ QQuickWebEngineAction *QQuickWebEngineView::action(WebAction action)
         break;
     case ReloadAndBypassCache:
         text = tr("Reload and Bypass Cache");
+        iconName = QStringLiteral("view-refresh");
         break;
     case Cut:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Cut);
-        iconName = QStringLiteral("Cut");
+        iconName = QStringLiteral("edit-cut");
         break;
     case Copy:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Copy);
+        iconName = QStringLiteral("edit-copy");
         break;
     case Paste:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Paste);
+        iconName = QStringLiteral("edit-paste");
         break;
     case Undo:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Undo);
+        iconName = QStringLiteral("edit-undo");
         break;
     case Redo:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::Redo);
+        iconName = QStringLiteral("edit-redo");
         break;
     case SelectAll:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::SelectAll);
+        iconName = QStringLiteral("edit-select-all");
         break;
     case PasteAndMatchStyle:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::PasteAndMatchStyle);
+        iconName = QStringLiteral("edit-paste");
         break;
     case OpenLinkInThisWindow:
         text = tr("Open link in this window");
@@ -1909,9 +1943,11 @@ QQuickWebEngineAction *QQuickWebEngineView::action(WebAction action)
         break;
     case ToggleMediaPlayPause:
         text = tr("Toggle Play/Pause");
+        iconName = QStringLiteral("media-playback-start");
         break;
     case ToggleMediaMute:
         text = tr("Toggle Mute");
+        iconName = QStringLiteral("audio-volume-muted");
         break;
     case DownloadMediaToDisk:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::DownloadMediaToDisk);
@@ -1921,31 +1957,38 @@ QQuickWebEngineAction *QQuickWebEngineView::action(WebAction action)
         break;
     case ExitFullScreen:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::ExitFullScreen);
+        iconName = QStringLiteral("view-fullscreen");
         break;
     case RequestClose:
         text = tr("Close Page");
+        iconName = QStringLiteral("window-close");
         break;
     case Unselect:
         text = tr("Unselect");
+        iconName = QStringLiteral("edit-select-none");
         break;
     case SavePage:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::SavePage);
+        iconName = QStringLiteral("document-save");
         break;
     case ViewSource:
         text = RenderViewContextMenuQt::getMenuItemName(RenderViewContextMenuQt::ContextMenuItem::ViewSource);
-        iconName = QStringLiteral("view-source");
         break;
     case ToggleBold:
         text = tr("&Bold");
+        iconName = QStringLiteral("format-text-bold");
         break;
     case ToggleItalic:
         text = tr("&Italic");
+        iconName = QStringLiteral("format-text-italic");
         break;
     case ToggleUnderline:
         text = tr("&Underline");
+        iconName = QStringLiteral("format-text-underline");
         break;
     case ToggleStrikethrough:
         text = tr("&Strikethrough");
+        iconName = QStringLiteral("format-text-strikethrough");
         break;
     case AlignLeft:
         text = tr("Align &Left");
@@ -1961,9 +2004,11 @@ QQuickWebEngineAction *QQuickWebEngineView::action(WebAction action)
         break;
     case Indent:
         text = tr("&Indent");
+        iconName = QStringLiteral("format-indent-more");
         break;
     case Outdent:
         text = tr("&Outdent");
+        iconName = QStringLiteral("format-indent-less");
         break;
     case InsertOrderedList:
         text = tr("Insert &Ordered List");
@@ -2206,7 +2251,8 @@ void QQuickContextMenuBuilder::addMenuItem(ContextMenuItem menuItem)
         m_view->d_ptr->ui()->addMenuSeparator(m_menu);
         return;
     }
-    action->d_ptr->setEnabled(isMenuItemEnabled(menuItem));
+    // Set enabled property directly with avoiding binding loops caused by its notifier signal.
+    action->d_ptr->m_enabled = isMenuItemEnabled(menuItem);
     m_view->d_ptr->ui()->addMenuItem(action, m_menu);
 }
 
